/** \file Refinement_MeshSize.h */

#ifndef REFINEMENT_MESHSIZE_H
#define REFINEMENT_MESHSIZE_H

using namespace AMDiS;

/** \brief
 * Refinement structure to perform local anisotropic refinement depending
 * on a refineFunction that defines the local refinement level to given
 * coordinates.
 **/
class RefinementMeshSizeCoords : public RefinementLevel<WorldVector<double>, double >
{
public:
  RefinementMeshSizeCoords(FiniteElemSpace *feSpace_, MeshRefinementFunction<WorldVector<double>, double >* refineFct_) :
    RefinementLevel<WorldVector<double>, double >(feSpace_, refineFct_) { };

  void markElements(Flag &markFlag) 
  {
    FUNCNAME("RefinementLevelCoords::markElements()");

    FixVec<WorldVector<double>, VERTEX> coords(mesh->getDim(), NO_INIT);
    WorldVector<double> center;

    bool elMarkRefine = false, elMarkCoarsen = false;

    Flag traverseFlag = Mesh::CALL_LEAF_EL | Mesh::FILL_COORDS;
    TraverseStack stack;
    ElInfo *elInfo = stack.traverseFirst(mesh, -1, traverseFlag);
    while (elInfo) {
      coords = elInfo->getCoords();
      center.set(0.0);
      for (int i = 0; i < coords.size(); ++i) {
	center += coords[i];
      }
      center *= 1.0 / static_cast<double>(coords.size());

      double refineH = (*refineFct)(center);
      double oldH = calcMeshSize(elInfo);
      elInfo->getElement()->setMark( calcMark(refineH, oldH) );

      elMarkRefine |= elInfo->getElement()->getMark() == 1;
      elMarkCoarsen |= elInfo->getElement()->getMark() == -1;
      elInfo = stack.traverseNext(elInfo);
    }
    
    markFlag = 0;
    if (elMarkRefine) 
      markFlag = 1;
    if (elMarkCoarsen) 
      markFlag |= 2;
  };
};


/** \brief
 * Refinement structure to perform local anisotropic refinement depending
 * on a refineFunction that defines the local refinement level to a given
 * value, like phase-value or distance-value.
 **/
class RefinementMeshSizeDOF : public RefinementLevel<double, double>
{
public:
  RefinementMeshSizeDOF(FiniteElemSpace *feSpace_, MeshRefinementFunction<double, double>* refineFct_, DOFVector<double> *vec_) :
    RefinementLevel<double, double>(feSpace_, refineFct_), 
    vec(vec_) { };

  void markElements(Flag &markFlag) 
  {
    FUNCNAME("RefinementLevelDOF::markElements()");

    double meanValue;

    ElementFunctionDOFVec<double> elFct(vec);
    DimVec<double> lambda(mesh->getDim(), NO_INIT);
    for (int i = 0; i < lambda.getSize(); ++i)
      lambda[i] = 1.0 / lambda.getSize();

    bool elMarkRefine = false, elMarkCoarsen = false;

    Flag traverseFlag = Mesh::CALL_LEAF_EL;
    TraverseStack stack;
    ElInfo *elInfo = stack.traverseFirst(mesh, -1, traverseFlag);
    while (elInfo) {
      elFct.setElInfo(elInfo);
      meanValue = elFct(lambda);

      double refineH = (*refineFct)(meanValue);
      double oldH = calcMeshSize(elInfo);
      elInfo->getElement()->setMark( calcMark(refineH, oldH) );


      elMarkRefine |= elInfo->getElement()->getMark() == 1;
      elMarkCoarsen |= elInfo->getElement()->getMark() == -1;
      elInfo = stack.traverseNext(elInfo);
    }
		
    markFlag = 0;
    if (elMarkRefine) 
      markFlag = 1;
    if (elMarkCoarsen) 
      markFlag |= 2;
  };

private:

  DOFVector<double>* vec;
};


/** \brief
 * Refinement structure to perform local anisotropic refinement depending
 * on a refineFunction that defines the local refinement level to a given
 * value, like phase-value or distance-value, and the center coords of the
 * element.
 **/
class RefinementMeshSizeCoordsDOF : public RefinementLevel<std::pair<WorldVector<double>, double>, double >
{
public:

  RefinementMeshSizeCoordsDOF(FiniteElemSpace *feSpace_, MeshRefinementFunction<std::pair<WorldVector<double>, double>, double >* refineFct_, DOFVector<double> *vec_) :
    RefinementLevel<std::pair<WorldVector<double>, double>, double >(feSpace_, refineFct_), vec(vec_) { };

  void markElements(Flag &markFlag) 
  {
    FUNCNAME("RefinementLevelCoordsDOF::markElements()");

    double meanValue;
    FixVec<WorldVector<double>, VERTEX> coords(mesh->getDim(), NO_INIT);
    WorldVector<double> center;

    ElementFunctionDOFVec<double> elFct(vec);
    DimVec<double> lambda(mesh->getDim(), NO_INIT);
    for (int i= 0; i<lambda.getSize(); ++i)
      lambda[i]= 1.0/lambda.getSize();

    bool elMarkRefine= false, elMarkCoarsen= false;

    Flag traverseFlag= Mesh::CALL_LEAF_EL | Mesh::FILL_COORDS;
    TraverseStack stack;
    ElInfo *elInfo= stack.traverseFirst(mesh, -1, traverseFlag);
    while (elInfo) {
      elFct.setElInfo(elInfo);
      meanValue = elFct(lambda);
      coords = elInfo->getCoords();
      center.set(0.0);
      for (int i = 0; i < coords.size(); ++i) {
        center += coords[i];
      }
      center *= 1.0 / static_cast<double>(coords.size());

      double refineH = (*refineFct)(std::make_pair<WorldVector<double>, double>(center, meanValue));
      double oldH = calcMeshSize(elInfo);
      elInfo->getElement()->setMark( calcMark(refineH, oldH) );

      elMarkRefine |= elInfo->getElement()->getMark() == 1;
      elMarkCoarsen |= elInfo->getElement()->getMark() == -1;
      elInfo = stack.traverseNext(elInfo);
    }
                
    markFlag = 0;
    if (elMarkRefine) 
      markFlag = 1;
    if (elMarkCoarsen) 
      markFlag |= 2;
  };

private:

  DOFVector<double>* vec;
};


/** \brief
 * Refinement structure to perform local anisotropic refinement depending
 * on a refineFunction that defines the local refinement level to a given
 * list of values, like phase-values or distance-values.
 **/
class RefinementMeshSizeDOFList : public RefinementLevel< std::vector<double>, double >
{
public:
  RefinementMeshSizeDOFList(
      FiniteElemSpace *feSpace_, 
      MeshRefinementFunction< std::vector<double>, double >* refineFct_, 
      DOFVector<double>* vec1_, 
      DOFVector<double>* vec2_ = NULL,
      DOFVector<double>* vec3_ = NULL) : 
    RefinementLevel< std::vector<double>, double >(feSpace_, refineFct_)
  {
    vecs.push_back(vec1_);
    if(vec2_ != NULL) vecs.push_back(vec2_);
    if(vec3_ != NULL) vecs.push_back(vec3_);			
  };

  void markElements(Flag &markFlag) 
  {
    FUNCNAME("RefinementLevelDOF::markElements()");

    const DOFAdmin* admin = feSpace->getAdmin();
    const BasisFunction *basFcts = feSpace->getBasisFcts();
    int numBasFcts = basFcts->getNumber();

    DegreeOfFreedom *localIndices = new DegreeOfFreedom[numBasFcts];
    FixVec<WorldVector<double>, VERTEX> coords(mesh->getDim(), NO_INIT);
    double meanValue;

    bool elMarkRefine = false, elMarkCoarsen = false;

    std::vector<ElementFunctionDOFVec<double>*> elFcts;
    for (unsigned i = 0; i < vecs.size(); ++i) {
      elFcts.push_back(new ElementFunctionDOFVec<double>(vecs[i]));
    }
    DimVec<double> lambda(mesh->getDim(), NO_INIT);
    for (int i= 0; i < lambda.getSize(); ++i)
      lambda[i] = 1.0 / lambda.getSize();

      Flag traverseFlag = Mesh::CALL_LEAF_EL;
      TraverseStack stack;
      ElInfo *elInfo = stack.traverseFirst(mesh, -1, traverseFlag);
      while(elInfo) {
	std::vector<double> meanValues;
	for (unsigned v = 0; v < vecs.size(); ++v) {
          elFcts[v]->setElInfo(elInfo);
	  meanValues.push_back( (*(elFcts[v]))(lambda) );
	}
	
        double refineH = (*refineFct)(meanValues);
        double oldH = calcMeshSize(elInfo);
        elInfo->getElement()->setMark( calcMark(refineH, oldH) );

	elMarkRefine |= elInfo->getElement()->getMark() == 1;
	elMarkCoarsen |= elInfo->getElement()->getMark() == -1;
	elInfo = stack.traverseNext(elInfo);
    }
      
    markFlag= 0;
    if(elMarkRefine) 
	    markFlag= 1;
    if(elMarkCoarsen) 
	    markFlag|= 2;
    
    delete [] localIndices;
  };
private:
	std::vector<DOFVector<double>*> vecs;
};


#endif // REFINEMENT_MESHSIZE_H

