/** \file DiffuseDomainFsi.h */

#ifndef DIFFUSE_DOMAIN_FSI_H
#define DIFFUSE_DOMAIN_FSI_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "ExtendedProblemStat.h"
#include "Helpers.h"
#include "POperators.h"

using namespace AMDiS;

/** \ingroup NavierStokes_TaylorHood
 * \brief
 * fully coupled fluid structure interaction using diffuse doamins
 * 
 * dt(u_j) + u*grad(u_j) - div(phase*sigma_f) - div((1-phase)*sigma_s) = 0; j=1...d
 * div(phase*u) = grad(phase)*u
 * (1-phase)*(dt(eta_j)+u*grad(eta_j)) = (1-phase)*u_j; j=1...d
 */
class DiffuseDomainFsi : public BaseProblem<ExtendedProblemStat>
{
public: // typedefs

  typedef BaseProblem<ExtendedProblemStat> super;

public: // methods

  DiffuseDomainFsi(const std::string &name_);
  ~DiffuseDomainFsi();
  
  virtual void initData();
  virtual void transferInitialSolution(AdaptInfo *adaptInfo);  
  virtual void initTimestep(AdaptInfo *adaptInfo);
  virtual void closeTimestep(AdaptInfo *adaptInfo);
  
  // === getting/setting methods ===
  
  DOFVector<double> *getPhase()
  {
    return phase;
  }
  
  double getEpsilon()
  {
    return epsilon;
  }

  DOFVector<WorldVector<double> >* getVelocity()
  {
    return velocity;
  }

  DOFVector<WorldVector<double> >* getDisplacement()
  {
    return displacement;
  }
  
  void setPhase(DOFVector<double>* phase_)
  {
    phase = phase_;
  }
  
  void setBcPhase(DOFVector<double>* phase_)
  {
    bcPhase = phase_;
  }
  
  void setEpsilon(double epsilon_)
  {
    epsilon = epsilon_;
  }

  void writeFiles(AdaptInfo* adaptInfo, bool force = false)
  {
    fileWriterPhase->writeFiles(adaptInfo, force);
    fileWriterVelocity->writeFiles(adaptInfo, force);
    fileWriterDisplacement->writeFiles(adaptInfo, force);
    super::writeFiles(adaptInfo, force);
  }

// protected: // methods

  virtual void fillOperators();
//   virtual void fillBoundaryConditions();
  virtual void addLaplaceTerm(int i);

protected: // variables
  
  double dbc_factor; // beta/epsilon^alpha
  double pressure_factor; // -(lambda+2/3*mu)
  double beta;
  double epsilon;
  double alpha;

  int nonLinTerm;

  double oldTimestep;

  // navier-stokes parameters
  double viscosity;
  double density;
  // elasticity parameters
  double mu;
  double lambda;
  double rho;
  
  DOFVector<double> *phase;
  DOFVector<double> *phaseInv;
  DOFVector<double> *phaseInvDisturbed;
  DOFVector<double> *bcPhase;

  DOFVector<WorldVector<double> >* velocity;

  void calcVelocity()
  {
    if (dow == 1)
      transformDOF(prob->getSolution()->getDOFVector(0), velocity, new AMDiS::Vec1WorldVec<double>);
    else if (dow == 2)
      transformDOF(prob->getSolution()->getDOFVector(0), prob->getSolution()->getDOFVector(1), velocity, new AMDiS::Vec2WorldVec<double>);
    else if (dow == 3)
      transformDOF(prob->getSolution()->getDOFVector(0), prob->getSolution()->getDOFVector(1), prob->getSolution()->getDOFVector(2), velocity, new AMDiS::Vec3WorldVec<double>);
  }
  
  DOFVector<WorldVector<double> >* displacement;

  void calcDisplacement()
  {
    if (dow == 1)
      transformDOF(prob->getSolution()->getDOFVector(dow+1), displacement, new AMDiS::Vec1WorldVec<double>);
    else if (dow == 2)
      transformDOF(prob->getSolution()->getDOFVector(dow+1), prob->getSolution()->getDOFVector(dow+2), displacement, new AMDiS::Vec2WorldVec<double>);
    else if (dow == 3)
      transformDOF(prob->getSolution()->getDOFVector(dow+1), prob->getSolution()->getDOFVector(dow+2), prob->getSolution()->getDOFVector(dow+3), displacement, new AMDiS::Vec3WorldVec<double>);
  }
  
  FileWriter* fileWriterPhase;
  FileVectorWriter* fileWriterVelocity;
  FileVectorWriter* fileWriterDisplacement;
};


struct ComponentExtractor : AbstractFunction<double, WorldVector<double> >
{
  ComponentExtractor(AbstractFunction<WorldVector<double>, WorldVector<double> >* fct_, int comp_, double factor_=1.0)
  : AbstractFunction<double, WorldVector<double> >(fct_->getDegree()),
    fct(fct_), 
    comp(comp_),
    factor(factor_) {}
  
  double operator()(const WorldVector<double>& x) const {
    WorldVector<double> result = (*fct)(x);
    return factor*result[comp];
  }
  
private:
  AbstractFunction<WorldVector<double>, WorldVector<double> >* fct;
  int comp;
  double factor;
};


struct VecAndComponentExtractor : BinaryAbstractFunction<double, double, WorldVector<double> >
{
  VecAndComponentExtractor(AbstractFunction<WorldVector<double>, WorldVector<double> >* fct_, int comp_, double factor_=1.0)
  : BinaryAbstractFunction<double, double, WorldVector<double> >(fct_->getDegree()+1),
    fct(fct_), 
    comp(comp_),
    factor(factor_) {}
  
  double operator()(const double& vec, const WorldVector<double>& x) const {
    WorldVector<double> result = (*fct)(x);
    return factor*vec*result[comp];
  }
  
private:
  AbstractFunction<WorldVector<double>, WorldVector<double> >* fct;
  int comp;
  double factor;
};

#endif // DIFFUSE_DOMAIN_FSI_H
