#include "LinearElasticityPhase.h"
#include "POperators.h"

using namespace std;
using namespace AMDiS;

LinearElasticityPhase::LinearElasticityPhase(const std::string &name_) :
  super(name_),
  mu(1.0),
  lambda(1.0),
  rho(1.0),
  epsilon(1.e-1),
  beta(1.0),
  alpha(3.0),
  displacement(NULL),
  phase(NULL),
  phaseOld(NULL),
  dirichletPhase(NULL),
  bcFct(NULL)
{  
  for (int i = 0; i < bcDOF.getSize(); ++i)
    bcDOF[i] = NULL;
  
  Parameters::get(name + "->mu", mu);
  Parameters::get(name + "->lambda", lambda);
  Parameters::get(name + "->rho", rho);
  Parameters::get(name + "->epsilon", epsilon);

  Parameters::get(name + "->beta", beta);
  Parameters::get(name + "->alpha", alpha);
  
  force.set(0.0);
  Parameters::get(name + "->force", force);
}


LinearElasticityPhase::~LinearElasticityPhase() 
{ FUNCNAME("LinearElasticityPhase::~LinearElasticityPhase()");

  if (displacement != NULL)
    delete displacement;
  delete fileWriter;
}


void LinearElasticityPhase::initData()
{ FUNCNAME("LinearElasticityPhase::initTimeInterface()");

  if (displacement == NULL)
    displacement = new DOFVector<WorldVector<double> >(getFeSpace(0), "displacement");
  phaseOld = new DOFVector<double>(getFeSpace(0), "phaseOld");
  fileWriter = new FileVectorWriter(name + "->displacement->output", getFeSpace()->getMesh(), displacement);

  super::initData();

  dbc_factor = beta/pow(epsilon, alpha);
  MSG_DBG("dbc_factor = %f\n", dbc_factor);
}


void LinearElasticityPhase::solveInitialProblem(AdaptInfo *adaptInfo) 
{ FUNCNAME("LinearElasticityPhase::solveInitialProblem()");

  super::solveInitialProblem(adaptInfo);
  
  for (size_t i = 0; i < 2*dow-1; i++)
    prob->getSolution()->getDOFVector(i)->set(0.0);
}


void LinearElasticityPhase::transferInitialSolution(AdaptInfo *adaptInfo)
{ FUNCNAME("LinearElasticityPhase::transferInitialSolution()");

  if (dow == 1)
    transformDOF(prob->getSolution()->getDOFVector(0), displacement, new AMDiS::Vec1WorldVec<double>);
  else if (dow == 2)
    transformDOF(prob->getSolution()->getDOFVector(0), prob->getSolution()->getDOFVector(1), displacement, new AMDiS::Vec2WorldVec<double>);
  else if (dow == 3)
    transformDOF(prob->getSolution()->getDOFVector(0), prob->getSolution()->getDOFVector(1), prob->getSolution()->getDOFVector(2), displacement, new AMDiS::Vec3WorldVec<double>);

  fileWriter->writeFiles(adaptInfo, false);
  writeFiles(adaptInfo, false);
  
  // initial parameters for detecting mesh changes
  oldMeshChangeIdx= getMesh()->getChangeIndex();
}


void LinearElasticityPhase::fillOperators()
{ FUNCNAME("LinearElasticityPhase::fillOperators()");
  MSG("LinearElasticityPhase::fillOperators()\n");

  // fill operators for prob
  for (size_t i = 0; i < dow; ++i) {
    /// dt(u) - v = 0
    /// ____________________________________________________________________________
    
    if (abs(rho) > DBL_TOL) {
      // < (1/tau)*u_i , psi >
      Operator *opTimeU = new Operator(getFeSpace(i+dow), getFeSpace(i));
      opTimeU->addTerm(new VecAtQP_ZOT(phase));
      prob->addMatrixOperator(*opTimeU, i+dow, i, getInvTau(), getInvTau());
      // < (1/tau)*u_i^old , psi >
      Operator *opTimeUOld = new Operator(getFeSpace(i+dow));
      opTimeUOld->addTerm(new Vec2AtQP_ZOT(phaseOld, prob->getSolution()->getDOFVector(i)));
      prob->addVectorOperator(*opTimeUOld, i+dow, getInvTau(), getInvTau());
    }
    // < v , psi >
    Operator *opV = new Operator(getFeSpace(i+dow), getFeSpace(i+dow));
    opV->addTerm(new Phase_ZOT(phase, -1.0));
    prob->addMatrixOperator(*opV, i+dow, i+dow);
    
    /// dt(v) - mu*div(grad(u)) - (mu+lambda)*grad(div(u)) = F
    /// ____________________________________________________________________________
 
    
    if (abs(rho) > DBL_TOL) {
      // < (1/tau)*v_i , psi >
      Operator *opTimeV = new Operator(getFeSpace(i), getFeSpace(i+dow));
      opTimeV->addTerm(new VecAtQP_ZOT(phase, new AMDiS::Factor<double>(rho)));
      prob->addMatrixOperator(*opTimeV, i, i+dow, getInvTau(), getInvTau());
      // < (1/tau)*v_i^old , psi >
      Operator *opTimeVOld = new Operator(getFeSpace(i));
      opTimeVOld->addTerm(new Vec2AtQP_ZOT(phase, prob->getSolution()->getDOFVector(i+dow), new AMDiS::MultScal<double>(rho)));
      prob->addVectorOperator(*opTimeVOld, i, getInvTau(), getInvTau());
    }
    
    // < grad(u_i) , grad(psi) >
    Operator *opDivGradU = new Operator(getFeSpace(i), getFeSpace(i));
    opDivGradU->addTerm(new Phase_SOT(phase, mu));
    prob->addMatrixOperator(*opDivGradU, i, i);

    // < d_j(u_j) , d_i(psi) >
    for (size_t j = 0; j < dow; j++) {
      Operator *opGradDivU = new Operator(getFeSpace(i), getFeSpace(j));
      opGradDivU->addTerm(new MatrixIJPhase_SOT(phase, i, j, mu+lambda));
      prob->addMatrixOperator(*opGradDivU, i, j);
    }
  
    // external force, i.e. gravitational force
    if (norm(force) > DBL_TOL) {
      Operator *opForce = new Operator(getFeSpace(i), getFeSpace(i));
      opForce->addTerm(new Phase_ZOT(phase, force[i]));
      prob->addVectorOperator(*opForce, i);
    }
  }
}


void LinearElasticityPhase::fillBoundaryConditions()
{ FUNCNAME("LinearElasticityPhase::fillBoundaryConditions()");

  /// neumann boundary condition
  for (size_t i = 0; i < dow; i++) {
    Operator *opNeumannForce = new Operator(getFeSpace(i), getFeSpace(i));
    if (bcDOF[i] != NULL) {
      opNeumannForce->addTerm(new VecAndGradVecAtQP_ZOT(bcDOF[i], phase, new NeumannBCDof));
    } else if (bcFct != NULL) {
      opNeumannForce->addTerm(new FctGradientCoords_ZOT(phase, new NeumannBCFct(new ComponentExtractor_(bcFct, i))));
    }
    prob->addVectorOperator(*opNeumannForce, i);
  }

  /// dirichlet boundary condition
  if (dirichletPhase != NULL) {
  for (size_t i = 0; i < dow; ++i) {
    /// dirichlet boundary condition
    /// beta/eps^alpha (1-phase)*(u_i - g_i), with g_i = 0
    Operator *opDbcPhaseInvU = new Operator(prob->getFeSpace(i), prob->getFeSpace(i));
    opDbcPhaseInvU->addTerm(new Simple_ZOT(dbc_factor));
    opDbcPhaseInvU->addTerm(new Phase_ZOT(dirichletPhase, -dbc_factor));
    prob->addMatrixOperator(*opDbcPhaseInvU, i, i);
  }
  }
}


void LinearElasticityPhase::initTimestep(AdaptInfo *adaptInfo)
{ FUNCNAME("LinearElasticityPhase::initTimestep()");

  super::initTimestep(adaptInfo);
  phaseOld->interpol(phase);
}


void LinearElasticityPhase::closeTimestep(AdaptInfo *adaptInfo)
{ FUNCNAME("LinearElasticityPhase::closeTimestep()");

  if (dow == 1)
    transformDOF(prob->getSolution()->getDOFVector(0), displacement, new AMDiS::Vec1WorldVec<double>);
  else if (dow == 2)
    transformDOF(prob->getSolution()->getDOFVector(0), prob->getSolution()->getDOFVector(1), displacement, new AMDiS::Vec2WorldVec<double>);
  else if (dow == 3)
    transformDOF(prob->getSolution()->getDOFVector(0), prob->getSolution()->getDOFVector(1), prob->getSolution()->getDOFVector(2), displacement, new AMDiS::Vec3WorldVec<double>);

  fileWriter->writeFiles(adaptInfo, false);
  writeFiles(adaptInfo, false);
}
