/** \file NavierStokes_TH_MultiPhase.h */

#ifndef NAVIER_STOKES_TAYLOR_HOOD_MULTIPHASE_H
#define NAVIER_STOKES_TAYLOR_HOOD_MULTIPHASE_H

#include "NavierStokes_TaylorHood.h"

#define LINKS 1
#define RECHTS 2
#define UNTEN 3
#define OBEN 4

using namespace AMDiS;

/** \ingroup NavierStokes_TaylorHood
 * \brief
 * Navier-Stokes multi-phase problem, using Taylor Hood elements
 * Standard implementation for two phases, but can be extended to
 * many more, by defining the multiPhase variable and overloading
 * the initTimestep, where densityPhase and viscosityPhase are defined
 * depending on the phases
 */
class NavierStokes_TH_MultiPhase : public NavierStokes_TaylorHood
{
public: // typedefs

  typedef NavierStokes_TaylorHood super;

public: // methods

  NavierStokes_TH_MultiPhase(const std::string &name_);

  /// deletes multiPhase, viscosityPhase and densityPhase
  ~NavierStokes_TH_MultiPhase();

  /** definition of constant multiPhase and const
   * viscosity/density DOFVectors,
   * initTimeInterface of super
   **/
  virtual void initData();

  /** initTimestep of super and
   * initialization of densityPhase and viscosityPhase
   **/
  virtual void initTimestep(AdaptInfo *adaptInfo);

  /** pointer to the multiPhase DOFVector, that
   * indicates the different phases. Will be initialized
   * in \ref initTimeInterface with const 1
   **/
  void setMultiPhase(DOFVector<double>* p) { multiPhase=p; }

protected: // methods

  virtual void fillOperators();
  virtual void addLaplaceTerm(int i);

protected: // variables

  ///viscosity of phase 1
  double viscosity1;
  
  ///viscosity of phase 2
  double viscosity2;
  
  ///density of phase 1
  double density1;
  
  ///density of phase 2
  double density2;

  /// phase dependent density
  DOFVector<double> *densityPhase;
  
  /// phase dependent viscosity
  DOFVector<double> *viscosityPhase;
  
  /// phase inticator
  DOFVector<double> *multiPhase;

};


/** linear interpolation between two values (like density, viscosity)
 *  using a phase-field variable in [-1,1]
 **/
class LinearInterpolation1 : public AbstractFunction<double, double>
{
public:

  LinearInterpolation1(double c1, double c2) :
    AbstractFunction<double, double>(1)
     { a = (c1-c2)/2.0; b = (c1+c2)/2.0; cmin=std::min(c1,c2); cmax=std::max(c1,c2);}
    
  double operator()(const double &phase) const { 
    double result = b+a*phase;
    if (result<cmin) result = cmin; 
    if (result>cmax) result = cmax;
    return result;
  }

private:
  double a,b,cmin,cmax;
};

/** linear interpolation between two values (like density, viscosity)
 *  using a phase-field variable in [0,1]
 **/
class LinearInterpolation0 : public AbstractFunction<double, double>
{
public:

  LinearInterpolation0(double val1_, double val2_) :
    AbstractFunction<double, double>(1),
    val1(val1_), val2(val2_) {}

  double operator()(const double &phase) const {    
    return phase*val1 + (1.0-phase)*val2;
  }

private:
  
  double val1;
  double val2;
};

#endif // NAVIER_STOKES_TAYLOR_HOOD_MULTIPHASE_H
