/** \file NavierStokes_TaylorHood.h */

#ifndef NAVIER_STOKES_TAYLOR_HOOD_H
#define NAVIER_STOKES_TAYLOR_HOOD_H

#include "AMDiS.h"
#include "Helpers.h"
#include "POperators.h"
#include "BoundaryFunctions.h"
#include "BaseProblem_RB.h"
#include "time/ExtendedRosenbrockStationary.h"
#include <boost/numeric/mtl/mtl.hpp>
#include <boost/numeric/mtl/utility/property_map.hpp>

// boundary numbers
#define LINKS 1
#define RECHTS 2
#define UNTEN 3
#define OBEN 4

using namespace AMDiS;

/** \ingroup NavierStokes_TaylorHood
 * \brief
 * Navier-Stokes problem, using Taylor Hood elements
 */
class NavierStokes_TaylorHood_RB : public BaseProblem_RB
{
public: // typedefs
  
  typedef BaseProblem_RB super;
  
public: // methods

  NavierStokes_TaylorHood_RB(const std::string &name_);

  /// if initialVelocityIsSet is true delete initialX and initialY
  ~NavierStokes_TaylorHood_RB();

  virtual void initData();
  virtual void solveInitialProblem(AdaptInfo *adaptInfo);

  virtual void transferInitialSolution(AdaptInfo *adaptInfo);  

  virtual void initTimestep(AdaptInfo *adaptInfo);
  /// Implementation of ProblemTimeInterface::closeTimestep().
  virtual void closeTimestep(AdaptInfo *adaptInfo);

  // === getting/setting methods ===

  /// set dimension of the macro-mesh
  void setDimension(WorldVector<double> dimension_) { dimension = dimension_; }

  DOFVector<WorldVector<double> >* getVelocity()
  {
    if (velocity == NULL)
      velocity = new DOFVector<WorldVector<double> >(getFeSpace(0), "velocity");
    return velocity;
  }

protected: // methods

  virtual void fillOperators();
  virtual void fillBoundaryConditions();
  
  virtual void addLaplaceTerm(int i);

protected: // variables

  AbstractFunction<double, WorldVector<double> > *initialX, *initialY;
  WorldVector<DOFVector<double>*> initialVel;
  std::vector<WorldVector<double> > dirichletPoints;

  DOFVector<WorldVector<double> >* velocity;
  
  bool forceDBC;
  bool initialVelocityIsSet;

  int laplaceType;
  int nonLinTerm;

  double minus1;
  double oldTimestep;
  double viscosity;
  double c;

  WorldVector<double> force;
  WorldVector<double> dimension;

  FileVectorWriter *fileWriter;
};
#endif // NAVIER_STOKES_TAYLOR_HOOD_H
