#include "AMDiS.h"
#include "CahnHilliard_.h"
#include "NavierStokes_TH_MultiPhase.h"
#include "CahnHilliardNavierStokes_.h"
#include "SignedDistFunctors.h"
#include "PhaseFieldConvert.h"
//#include "CahnHilliard_.h"

#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
  #include "parallel/PetscSolverCahnHilliard.h"
  #include "parallel/PetscSolverCahnHilliard2.h"
  #include "parallel/PetscSolverNavierStokes.h"
#endif

#include "boost/date_time/posix_time/posix_time.hpp"

using namespace AMDiS;
using namespace boost::posix_time;



class CahnHilliardPC : public CahnHilliard_
{
public:
  CahnHilliardPC(std::string name) : CahnHilliard_(name), delta(1.0)
  { }
 
  void initData()
  {   
    #ifdef HAVE_PARALLEL_DOMAIN_AMDIS
    string initFileStr = name + "->space->solver", solverType = "";
    Parameters::get(initFileStr, solverType);
    if (solverType == "petsc-ch") {
      PetscSolverCahnHilliard* solver = dynamic_cast<PetscSolverCahnHilliard*>(prob->getSolver());
      if (solver)
	solver->setChData(&eps, &delta, prob->getSolution());	
    } else if (solverType == "petsc-ch2") {
      PetscSolverCahnHilliard2* solver = dynamic_cast<PetscSolverCahnHilliard2*>(prob->getSolver()); 
      if (solver)
	solver->setChData(&eps, &delta);
    }
    #endif
  }
  
  virtual void setTime(AdaptInfo* adaptInfo) 
  {
    super::setTime(adaptInfo);
    delta = gamma*adaptInfo->getTimestep();
  }
  
  double delta;
};





class NavierStokes_TaylorHoodPC : public NavierStokes_TH_MultiPhase
{
public:
  NavierStokes_TaylorHoodPC(std::string name) : NavierStokes_TH_MultiPhase(name)
  { }
 
 
 void initData()
  {   
    NavierStokes_TH_MultiPhase::initData(); /// warum geht super::initData() nicht ? 
    #ifdef HAVE_PARALLEL_DOMAIN_AMDIS
    string initFileStr = name + "->space->solver", solverType = "";
    Parameters::get(initFileStr, solverType);
    if (solverType == "petsc-navierstokes") {
      PetscSolverNavierStokes *s = dynamic_cast<PetscSolverNavierStokes*>(prob->getPetscSolver());
      if (s) {
	  s->setStokesData( getInvTau(), prob->getSolution(), &viscosity1, &viscosity2, &density1, &density2);
	  s->setPhase(multiPhase);
      }
    } 
    #endif    
  }
  
  void initPhasefield()
  {   
    #ifdef HAVE_PARALLEL_DOMAIN_AMDIS
    string initFileStr = name + "->space->solver", solverType = "";
    Parameters::get(initFileStr, solverType);
    if (solverType == "petsc-navierstokes") {
      PetscSolverNavierStokes *s = dynamic_cast<PetscSolverNavierStokes*>(prob->getPetscSolver());
      if (s) {
	  s->setPhase(multiPhase);
      }
    } 
    #endif    
  }
  
};




class MyNavierStokes_PC : public NavierStokes_TaylorHoodPC
{
public:
  MyNavierStokes_PC(std::string name_) : NavierStokes_TaylorHoodPC(name_) {}
  
protected:
  void fillBoundaryConditions()
  { FUNCNAME("MyNavierStokes_PC::fillBoundaryConditions()");
    
    DOFVector<double> *zeroDOF = new DOFVector<double>(getFeSpace(0), "zero");
    zeroDOF->set(0.0);
    size_t dow = Global::getGeo(WORLD);

    /// at rigid wall: no-slip boundary condition  
    // oben-unten
    getProblem()->addDirichletBC(2, 0, 0, zeroDOF);
    getProblem()->addDirichletBC(2, 1, 1, zeroDOF);
    // links-rechts
    getProblem()->addDirichletBC(1, 0, 0, zeroDOF); 
  }
  
  
};


class MyNavierStokes: public NavierStokes_TH_MultiPhase
{
public:
  MyNavierStokes(std::string name_) : NavierStokes_TH_MultiPhase(name_) {}
  
protected:
  void fillBoundaryConditions()
  { FUNCNAME("MyNavierStokes::fillBoundaryConditions()");
    
    DOFVector<double> *zeroDOF = new DOFVector<double>(getFeSpace(0), "zero");
    zeroDOF->set(0.0);
    size_t dow = Global::getGeo(WORLD);

    /// at rigid wall: no-slip boundary condition  
    // oben-unten
    getProblem()->addDirichletBC(2, 0, 0, zeroDOF);
    getProblem()->addDirichletBC(2, 1, 1, zeroDOF);
    // links-rechts
    getProblem()->addDirichletBC(1, 0, 0, zeroDOF); 
  }
  
  
};



int main(int argc, char** argv)
{ FUNCNAME("main");

  AMDiS::init(argc, argv);
  
  #ifdef HAVE_PARALLEL_DOMAIN_AMDIS
      CreatorMap<OEMSolver>::addCreator("petsc-ch", new PetscSolverCahnHilliard::Creator);
      CreatorMap<OEMSolver>::addCreator("petsc-ch2", new PetscSolverCahnHilliard2::Creator);
  #endif

  #ifdef HAVE_PARALLEL_DOMAIN_AMDIS      
    CahnHilliardPC chProb("ch");
    MyNavierStokes_PC nsProb("ns");
    CahnHilliardNavierStokes_<CahnHilliardPC,MyNavierStokes_PC> mainProb("main", &chProb, &nsProb);
  #else
    CahnHilliard_ chProb("ch");
    MyNavierStokes_PC nsProb("ns");
    CahnHilliardNavierStokes_<CahnHilliard_,MyNavierStokes> mainProb("main", &chProb, &nsProb);
  #endif
  
  
  
    
  // Adapt-Infos
  AdaptInfo adaptInfo("adapt", mainProb.getNumComponents());
  mainProb.initialize(&adaptInfo);
  
  nsProb.setMultiPhase(chProb.getSolution()->getDOFVector(1));
  #ifdef HAVE_PARALLEL_DOMAIN_AMDIS      
    nsProb.initPhasefield();
  #endif
  // adaption loop - solve ch-prob and ns-prob
  AdaptInstationary adaptInstat("adapt", mainProb, adaptInfo, mainProb, adaptInfo);


  
  ptime start_time = microsec_clock::local_time();
  int error_code = adaptInstat.adapt(); 
  time_duration td = microsec_clock::local_time()-start_time;

  MSG("elapsed time= %d sec\n", td.total_seconds());

  AMDiS::finalize();

  return error_code;
};
