/** \file ElasticityNavierStokes.h */

#ifndef ELASTICITY_NAVIER_STOKES_H
#define ELASTICITY_NAVIER_STOKES_H

// coupling structures
#include "CouplingIterationInterface.h"
#include "CouplingTimeInterface.h"
#include "CouplingProblemStat.h"

// structures for local refinement
#include "Refinement.h"
#include "MeshFunction_Level.h"

#include "NavierStokesPhase_TaylorHood.h"
#include "LinearElasticity.h"
#include "navierStokes.h"
#include "SignedDistFunctors.h"
#include "PhaseFieldConvert.h"

using namespace AMDiS;

/**
  * \ingroup Problem 
  *
  * \brief
  */
template<typename Elasticity_Type1 = LinearElasticity,
		 typename Elasticity_Type2 = StandardBaseProblem,
		 typename NS_Type = NavierStokesPhase_TaylorHood>
class ElasticityNavierStokes : public CouplingIterationInterface,
                                 public CouplingTimeInterface,
                                 public CouplingProblemStat
{
public:

  ElasticityNavierStokes(std::string name_, Elasticity_Type1 *elastProb_, Elasticity_Type2 *elastProb2_, NS_Type *nsProb_)
  : CouplingProblemStat(name_),
    elastProb(elastProb_),
    elastProb2(elastProb2_),
    nsProb(nsProb_),
    refFunction1(NULL),
    refFunction2(NULL),
    refinement1(NULL),
    refinement2(NULL)
  {
    dow = Global::getGeo(WORLD);
  }


  ~ElasticityNavierStokes() {
    if (refFunction1 != NULL)
      delete refFunction1;
    if (refinement1 != NULL)
      delete refinement1;
    if (refFunction2 != NULL)
      delete refFunction2;
    if (refinement2 != NULL)
      delete refinement2;

    for (int i = 0; i < Global::getGeo(WORLD); i++) {
      delete parametricCoords1[i];
      delete parametricCoords2[i];
	}

    delete parametric1;
    delete parametric2;
  }


  /**
   * Add the problems to the iterationInterface, timeInterface and couplingProblemStat.
   * As a consequence all problem can be initialized one after another and in the
   * adaption loop they are solved in rotation.
   * At the end the problems are filled with operators and coupling operators as well as
   * boundary conditions are added.
   *
   * In the adaption loop the problems are solved the same order as they are added to the
   * iterationInterface in this method. This order can be changed manually in the oneIteration
   * method.
   **/
  void initialize(AdaptInfo *adaptInfo)
  {
    for (size_t i = 0; i < elastProb->getNumProblems(); i++)
      addProblem(elastProb->getProblem(i));
    for (size_t i = 0; i < elastProb2->getNumProblems(); i++)
      addProblem(elastProb2->getProblem(i));
    for (size_t i = 0; i < nsProb->getNumProblems(); i++)
      addProblem(nsProb->getProblem(i));

    addIterationInterface(elastProb);
    addIterationInterface(elastProb2);
    addIterationInterface(nsProb);

    addTimeInterface(elastProb);
    addTimeInterface(elastProb2);
    addTimeInterface(nsProb);

    CouplingProblemStat::initialize(INIT_ALL | INIT_EXACT_SOLUTION);
// 	elastProb->initialize(INIT_ALL);
// 	elastProb2->initialize(INIT_ALL);
// 	nsProb->initialize(INIT_ALL - CREATE_MESH, elastProb2->getProblem(), INIT_MESH);
    dim = elastProb->getMesh()->getDim();

    initData();
    elastProb->initData();
    elastProb2->initData();
    nsProb->initData();
    fillCouplingBoundaryConditions();

    elastProb->fillOperators();
    elastProb2->fillOperators();
    nsProb->fillOperators();

    elastProb->fillBoundaryConditions();
    elastProb2->fillBoundaryConditions();
    nsProb->fillBoundaryConditions();
  }

  
  void initData()
  { MSG("initData()\n");

    // ===== create coords vector =====
    for (int i = 0; i < Global::getGeo(WORLD); i++) {
      parametricCoords1[i] = new DOFVector<double>(elastProb->getFeSpace(0),
						  "parametric1 coords "+boost::lexical_cast<std::string>(i));
      parametricCoords2[i] = new DOFVector<double>(elastProb2->getFeSpace(0),
						  "parametric2 coords "+boost::lexical_cast<std::string>(i));
    }

    // ===== create parametric object =====
    parametric1 = new ParametricFirstOrder(&parametricCoords1);
    parametric2 = new ParametricFirstOrder(&parametricCoords2);
  }


  void fillCouplingBoundaryConditions()
  { MSG("fillCouplingBoundaryConditions()\n");
    elastProb->setNormalStress(nsProb->getNormalStress());
    elastProb2->setDeformation(elastProb->getDeformation());
    nsProb->setDeformationVelocity(elastProb->getDeformationVelocity());
  }


  /**
   * Solves the initial problems.Before this is done the mesh is refined by the Refinement-class
   * This provides local refinement functions for phaseField-refinement or signed-distance-refinement.
   * See MeshFunction_Level.h for some details.
   **/ 
  void solveInitialProblem(AdaptInfo *adaptInfo)
  { MSG("solveInitialProblem()\n");
  
  
  
    size_t nVertices = 0;
    WorldVector<double> c,c2;
    Parameters::get("obstacle->num vertices",nVertices);
    std::vector<WorldVector<double> > v(nVertices,c);
    for (size_t i = 0; i < nVertices; i++)
      Parameters::get("obstacle->vertex["+boost::lexical_cast<std::string>(i)+"]",v[i]);
    v.push_back(v[0]);
    
    obstacle = new Polygon(v);

    Parameters::get("flag->num vertices",nVertices);
    std::vector<WorldVector<double> > v2(nVertices,c);
    for (size_t i = 0; i < nVertices; i++)
      Parameters::get("flag->vertex["+boost::lexical_cast<std::string>(i)+"]",v2[i]);
    v2.push_back(v2[0]);
    
    flag = new Polygon(v2);

    Parameters::get("connection->num vertices",nVertices);
    std::vector<WorldVector<double> > v3(nVertices,c);
    for (size_t i = 0; i < nVertices; i++)
      Parameters::get("connection->vertex["+boost::lexical_cast<std::string>(i)+"]",v3[i]);
    v3.push_back(v3[0]);
    
    signedDistFct = new MinWrapper(new Polygon(v2), new MinWrapper(obstacle,flag));

    refFunction1 = new SignedDistRefinement(elastProb->getMesh());
    refinement1 = new RefinementLevelCoords2(
      elastProb->getFeSpace(0),
      refFunction1,
      signedDistFct);
	
    refFunction2 = new SignedDistRefinement(elastProb2->getMesh());
    refinement2 = new RefinementLevelCoords2(
      elastProb2->getFeSpace(0),
      refFunction2,
      signedDistFct);

    // initial refinement
    refinement1->refine(10);
    refinement2->refine(10);
    flag->refine(10);

    // solve all initial problems of the problems added to the CouplingTimeInterface
    CouplingTimeInterface::solveInitialProblem(adaptInfo);
	
	DOFVector<WorldVector<double> > coords1(parametricCoords1[0]->getFeSpace(), "coords");
	parametricCoords1[0]->getFeSpace()->getMesh()->getDofIndexCoords(parametricCoords1[0]->getFeSpace(), coords1);
	DOFVector<WorldVector<double> > coords2(parametricCoords2[0]->getFeSpace(), "coords");
	parametricCoords2[0]->getFeSpace()->getMesh()->getDofIndexCoords(parametricCoords2[0]->getFeSpace(), coords2);
    for (int i = 0; i < Global::getGeo(WORLD); i++) {
	  transformDOF(&coords1, parametricCoords1[i], new Component(i));
	  transformDOF(&coords2, parametricCoords2[i], new Component(i));
	}
	
    // ===== enable parametric traverse =====
    elastProb->getMesh()->setParametric(parametric1);
    elastProb2->getMesh()->setParametric(parametric2);
  }


  /**
   * Called at the end of each timestep.
   * If the phase-field has changed the mesh is updated by the refinement-method
   **/
  void closeTimestep(AdaptInfo *adaptInfo)
  { MSG("closeTimestep()\n");
  
    CouplingTimeInterface::closeTimestep(adaptInfo);

//     flag->move(elastProb->getDeformation());
    updateMesh(elastProb->getDeformation(), parametricCoords1, parametric1);
    updateMesh(elastProb2->getDeformation(), parametricCoords2, parametric2);
//     refinement1->refine(5);
//     refinement2->refine(5);
  }

  void updateMesh(WorldVector<DOFVector<double>*> vec, WorldVector<DOFVector<double>*>& parametricCoords, ParametricFirstOrder* parametric)
  {
    FUNCNAME("ParametricSphere::buildAfterCoarsen()");
    MSG("calculation of parametric coordinates\n");
    const FiniteElemSpace* feSpace = vec[0]->getFeSpace();
    int dim = feSpace->getMesh()->getDim();
    int dow = Global::getGeo(WORLD);
    WorldVector<double> newCoords;

    DegreeOfFreedom dof;
    WorldVector<double> x;

    const BasisFunction *basFcts = feSpace->getBasisFcts();
    int numBasFcts = basFcts->getNumber();
    std::vector<DegreeOfFreedom> localIndices(numBasFcts);
    DOFAdmin *admin = feSpace->getAdmin();

    // ===== disable parametric traverse =====
    feSpace->getMesh()->setParametric(NULL);

    std::map<DegreeOfFreedom, bool> visited;

    TraverseStack stack;
    ElInfo *elInfo = stack.traverseFirst(feSpace->getMesh(), -1,
					Mesh::CALL_LEAF_EL | Mesh::FILL_COORDS);
    while (elInfo) {
      basFcts->getLocalIndices(elInfo->getElement(), admin, localIndices);
      for (int i = 0; i < dim + 1; i++) {
	dof = localIndices[i];
	if (!visited[dof]) {
	  for (int j = 0; j < dow; j++)
	    (*(parametricCoords[j]))[dof] += (*vec[j])[dof];

	  visited[dof] = true;
	}
      }
      elInfo = stack.traverseNext(elInfo);
    }

    // ===== enable parametric traverse =====
    feSpace->getMesh()->setParametric(parametric);
  }

protected:

  Elasticity_Type1 *elastProb;	// LinearElasticity baseProblem
  Elasticity_Type2 *elastProb2;	// LinearElasticity baseProblem
  NS_Type *nsProb;	// Navier-Stokes baseProblem

  /// DOFVector storing parametric coordinates.
  WorldVector<DOFVector<double>*> parametricCoords1;
  WorldVector<DOFVector<double>*> parametricCoords2;

  /// Parametrizes vertex coordinates while mesh traversal.
  ParametricFirstOrder *parametric1;
  ParametricFirstOrder *parametric2;

  Polygon* flag;
  Polygon* obstacle;
  AbstractFunction<double, WorldVector<double> >* signedDistFct;

  SignedDistRefinement* refFunction1;
  SignedDistRefinement* refFunction2;
  RefinementLevelCoords2* refinement1;
  RefinementLevelCoords2* refinement2;

  unsigned dim;		// dimension of the mesh
  unsigned dow;		// dimension of the world
};


#endif // CAHN_HILLIARD_NAVIER_STOKES_H
