// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ExtendedRosenbrockStationary.h */

#ifndef MY_ROSENBROCKSTATIONARY_H
#define MY_ROSENBROCKSTATIONARY_H

#include "AMDiS_fwd.h"
#include "ExtendedProblemStat.h"
#include "SystemVector.h"
#include "time/RosenbrockMethod.h"

#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
#ifdef HAVE_PARALLEL_MTL4
#include "parallel/Mtl4Solver.h"
#else
#include "parallel/PetscProblemStat.h"
#endif
#endif

using namespace AMDiS;

  struct MyRosenbrockBoundary 
  {
    MyRosenbrockBoundary(AbstractFunction<double, WorldVector<double> > * fct_,
			 AbstractFunction<double, WorldVector<double> > * fctDt_,
			 DOFVector<double> *vec_,
			 int row_,
			 int col_      
    ) : fct(fct_), fctDt(fctDt_), vec(vec_), row(row_), col(col_) {}
    
    AbstractFunction<double, WorldVector<double> > *fct; // f(t,x)
    AbstractFunction<double, WorldVector<double> > *fctDt; // dt[f](t,x)
    
    DOFVector<double> *vec;

    int row;
    
    int col;
  };


  class ExtendedRosenbrockStationary : public ExtendedProblemStat
  {
  public: // typedefs
    typedef ExtendedProblemStat super;

  public:
    ExtendedRosenbrockStationary(std::string name, int componentShift_ = 0)
      : super(name),
	componentShift(componentShift_),
	first(true),
	fixedTimestep(false),
	minusOne(-1.0),
	tauPtr(NULL),
	tauGamma(NULL),
	minusTauGamma(NULL),	
	invTauGamma(NULL),
	minusInvTauGamma(NULL),
	stageTime(0.0)
    {}

    DOFVector<double>* getUnVec(int i)
    {
      return unVec->getDOFVector(i);
    }

    DOFVector<double>* getStageSolution(int i)
    {
      return stageSolution->getDOFVector(i);
    }

    DOFVector<double>* getTimeRhsVec(int i)
    {
      return timeRhsVec->getDOFVector(i);
    }

    void acceptTimestep();

    void buildAfterCoarsen(AdaptInfo *adaptInfo, Flag flag,
			   bool asmMatrix, bool asmVector);

    void solve(AdaptInfo *adaptInfo, bool a = true, bool b = false);

    void setRosenbrockMethod(RosenbrockMethod *method)
    {
      rm = method;
      init();
    }
    
    void setFixedTimestep(bool fixed) { fixedTimestep = fixed; }

    void reset()
    {
      first = true;
      stageSolution->set(0.0);
      unVec->set(0.0);
      for (int i = 0; i < rm->getStages(); i++)
	stageSolutions[i]->set(0.0);
    }

    void addOperator(Operator &op, int row, int col, 
		     double *factor = NULL, double *estFactor = NULL);

    void addJacobianOperator(Operator &op, int row, int col, 
			     double *factor = NULL, double *estFactor = NULL);

    void addTimeOperator(int i, int j, double factor = 1.0);

    void setTau(double *ptr)
    {
      tauPtr = ptr;
    }

    void setTauGamma(double *ptr0, double *ptr1, double *ptr2, double *ptr3, double *ptr4)
    {
      tauGamma = ptr0;
      minusTauGamma = ptr1;
      invTauGamma = ptr2;
      minusInvTauGamma = ptr3;
      oldTime = ptr4;
    }

    double* getTauGamma()
    {
      return tauGamma;
    }

     double* getMinusTauGamma()
    {
      return minusTauGamma;
    }

    double* getInvTauGamma()
    {
      return invTauGamma;
    }

    double* getMinusInvTauGamma()
    {
      return minusInvTauGamma;
    }
    
    double getStageTime()
    {
      return stageTime;
    }
    
    double* getOldTimePtr()
    {
      return oldTime;
    }
    
    double* getStageTimePtr()
    {
      return &stageTime;
    }
    
    double* getTauGammaI()
    {
      return &tauGammaI;
    }

   /// Adds a Dirichlet boundary condition, where the rhs is given by an 
    /// abstract function.
    void addDirichletBC(BoundaryType type, int row, int col,
			AbstractFunction<double, WorldVector<double> > *b,
			AbstractFunction<double, WorldVector<double> > *bDt = NULL);

    /// Adds a Dirichlet boundary condition, where the rhs is given by a DOF
    /// vector.
    void addDirichletBC(BoundaryType type, int row, int col,
			DOFVector<double> *vec)
    {
      FUNCNAME("ExtendedRosenbrockStationary::addDirichletBC()");

      ERROR_EXIT("Not yet supported!\n");
    }

    /// special boundary conditions defined in ExtendedProblemStat

    void addSingularDirichletBC(WorldVector<double> &pos, int row, int col,
				AbstractFunction<double, WorldVector<double> > &fct);

    void addSingularDirichletBC(DegreeOfFreedom idx, int row, int col,
				AbstractFunction<double, WorldVector<double> > &fct);

    void addImplicitDirichletBC(AbstractFunction<double, WorldVector<double> > &signedDist, int row, int col,
				AbstractFunction<double, WorldVector<double> > &fct);
    
    void addImplicitDirichletBC(DOFVector<double> &signedDist, int row, int col,
				AbstractFunction<double, WorldVector<double> > &fct);

    void addManualDirichletBC(BoundaryType nr, AbstractFunction<bool, WorldVector<double> >* meshIndicator,
			      int row, int col,
			      AbstractFunction<double, WorldVector<double> > &fct);

    void addManualDirichletBC(AbstractFunction<bool, WorldVector<double> >* meshIndicator,
			      int row, int col,
			      AbstractFunction<double, WorldVector<double> > &fct);

  protected:
    void init();

  protected:    
    RosenbrockMethod *rm;

    SystemVector *stageSolution, *unVec, *timeRhsVec, *newUn, *tmp, *lowSol, *timeDrivative;

    std::vector<SystemVector*> stageSolutions;

    int componentShift;
    
    bool first;
    bool fixedTimestep;

    double minusOne;

    double *tauPtr;

    double *tauGamma, *minusTauGamma, *invTauGamma, *minusInvTauGamma;
    
    double stageTime; // t_n + alpha_i*tau
    double* oldTime;
    double tauGammaI; // tau*gamma_i

    std::vector<MyRosenbrockBoundary> boundaries;
  };


#endif // MY_ROSENBROCKSTATIONARY_H
