
#ifndef AMDIS_EXTENSIONS_TUTORIAL_INCLUDE
#define AMDIS_EXTENSIONS_TUTORIAL_INCLUDE

// This file contains no source code but only documentation.

/*! \mainpage AMDiS Extensions

\author Simon Praetorius

Many things can be realized using AMDiS today, but some things
are really complicated to implement. Other common tasks need a lot
of coding to be solved.

In the AMDiS-Extensions a lot of standard task are realized and 
implemented. This manual should give you a short introduction
to some of the features. The extensions are work in progress, so
the signature and interface of the methods and classes can change
from one version to the next. Nevertheless it builds a set of
very useful tools are used in every day work by several projects
today.

The manual still not covers all features of the AMDiS-Extensions.
But it should give you enough information to get started.

- \subpage intro
- \subpage data_on_unstructured_grid
- \subpage mesh_refinement 
- \subpage boundary_conditions 
- \subpage tools
- \subpage base_problems


*/

//-----------------------------------------------------------

/*! \page intro Introduction




To use the extensions you have to comile AMDiS with ENABLE_EXTENSIONS set
in the ccmake dialog. Then you have to specify the path to the extensions
directory (normally trunk/extensions).

An other type of extensions are a set of base problems. Often used physical
problems are outsourced in classes derived from BaseProblem class. Examples
are 

- Phase-Field Crystal (PFC)
- Cahn-Hilliard
- Navier-Stokes
  - Pressure-Correction method using Chorin projection
  - Taylor-Hood elements for the incompressible Navier-Stokes eq.
  - Extensions to Diffuse-Domain, Multiphase, Rosenbrock-time-discretization
    implementations for both types
- Linear elasticity
- (Fluid-Structure interaction)

The base problems must be compiled with AMDiS by activating the ENABLE_BASE_PROBLEMS
flag after the ENABLE_EXTENSIONS flag is set.

*/

//-----------------------------------------------------------


/*! \page data_on_unstructured_grid Handling data on unstructured grids

In AMDiS grid data is stored in a structure called DOFVector. It holds values with respect to an underlying
mesh and finite element space. Currently only Lagrange FEs of order 1-4 are implemented. For now we restrict
ourself to linear polynomial elements in a mesh with intervals, triangles or tetrahedron (in 1d, 2d, or 
3d respectively).

In general DOFVectors&lt; T &gt; are mappings \f$ dofindex\mapsto T \f$, where dofindex is the index of a
degree of freedom (for linear elements, just the nodes in the mesh) and <c>T</c> is the value type of the DOFVector,
typically <c>T=double</c>, or <c>T=WorldVector&lt;double&gt;</c>. The evaluation or access of a DOFVector at
coordinates is in general an expensive action, so the question is how to assign function values to a DOFVector, 
perform elementary arithmetic operations, or reduction operations, like integration, on this structure.

The following sections give an interaction to DOFVectors and how to deal with it:
- Generate and fill a DOFVector
- Elementary arithmetic operations with DOFVectors
- Reduction of DOFVectors to scalars
  - Integrate on subdomains
- Gradients of DOFVectors
  - Symmetric gradients
- Evaluation of DOFVectors at coordinates

\section fill_dof_vector Generate and fill a DOFVector

The first step is to generate a DOFVector <c>u</c> and fill it with some values \f$ f(x) \f$, e.g.

\f{eqnarray*}{
u(x) &:=& \sin(\|x\|) \\
v(x) &:=& \big(\|x\|\cdot\sin(x_0),\; \|x\|\cdot\sin(x_1),\;\ldots\big)^\top
\f}

Here <c>u=DOFVector&lt; double &gt;</c> and <c>v=DOFVector&lt; WorldVector&lt;double&gt; &gt;</c>. The constructor of a DOFVector
gets two arguments, first the corresponding FeSpace the DOFVector should be build on, and second a name.

\code
DOFVector< double > u(feSpace, "u");
DOFVector< WorldVector<double> > v(feSpace, "v");
\endcode

We can assign a constant value by the simple call <c>u.set(value)</c>. To assign function values one can interpolate an 
AbstractFunction to the DOFVector. For our examples, we get

\code
struct U : AbstractFunction< double, WorldVector<double> >
{
  double operator()(const WorldVector<double>& x) const
  {
    return sin(norm(x));
  }
};
struct V : AbstractFunction< WorldVector<double>, WorldVector<double> >
{
  WorldVector<double> operator()(const WorldVector<double>& x) const
  {
    WorldVector<double> result;
    for (int i = 0; i < x.getSize(); i++)
      result[i] = norm(x)*sin(x[i]);
    
    return result;
  }
};
...
u.interpol(new U);
v.interpol(new V);
\endcode

To assign the values of an other DOFVector <c>u2</c> to the vector <c>u</c>, there are several methods, depending on the 
underlying grids and feSpaces of the two DOFVectors. If they are all the same, you can simply call

\code
u.copy(u2);
\endcode

If the macro-meshes are the same, one can call

\code
u.interpol(u2);
\endcode

and if the DOFVectors live on completely different geometric structures, i.e. different feSpaces and different meshes, one
has to use a method defined in Views.h

\code
u.interpol_coords(u2);
\endcode

\section calc_with_dof_vectors Elementary arithmetic operations with DOFVectors

Very often in postprocessing steps of simulations one has to calculate with DOFVectors, e.g. the sum a solution components,
or a double-well function from a phasefield solution, i.e. a phasefield for a signed distance function that describes a circle,
is defined by

\f[ p(x) := \frac{1}{2}\Big(1 - \tanh\big(3/\epsilon \cdot (\|x\|-radius)\big)\Big) \f]

and the double-well can be calculated by

\f[ B(p)(x) := \frac{36}{\epsilon}\,p(x)^2\big(1-p(x)\big)^2 \f]

To apply these operations to DOFVectors you have to use the transformDOF methods, defined in TransformDof.h. These methods have a
lot of different argument combinations. The general signature is: at first the input containers, than the output DOFVector and at 
last a functor (AbstractFunction, BinaryAbstractFunction, TertiaryAbstractFunction), that describes the operations to do with
the DOFVectors. The number of input arguments depend on the number of arguments in the Functor. Implemented is the combination of
one, two or three input containers of the type DOFVector or scalar. There are several predefined functor in the file Functors.h, 
e.g. Id, Const, Add, Subtract, Mult, Max, Min, ..., but for our double-well we have to define an own functor:

\code
struct PhaseField : AbstractFunction< double, WorldVector<double> >
{
  double operator()(const WorldVector<double>& x) const
  {
    static const double epsilon = 0.01;
    static const double radius = 0.3;
    return 0.5 * (1.0 - tanh(3/epsilon * (norm(x) - radius)));
  }
};
struct DoubleWell : AbstractFunction< double, double >
{
  double operator()(const double& p) const
  {
    static const double epsilon = 0.01;
    return 36.0/epsilon * sqr(p) * sqr(1 - p);
  }
};
...
DOFVector< double > p(feSpace, "p");
DOFVector< double > B(feSpace, "B(p)");
p.interpol(new PhaseField);
transformDOF(&p, &B, new DoubleWell);
\endcode

You can also combine DOFVectors with different value types, e.g. calculate pointwise the norm of a 
<c>DOFVector&lt; WorldVector&lt;double&gt; &gt;</c>, like the DOFVector v defined in the last section

\code
DOFVector< double > norm_v(feSpace, "|v|");
transformDOF(&v, &norm_v, new AMDiS::Norm2< WorldVector<double> >);
\endcode

\section dof_vector_reduction Reduction of DOFVectors to scalars

Typically one reduces a vector, to calculate a norm of a vector, e.g. L<sub>1</sub>-Norm, L<sub>2</sub>-Norm, 
L<sub>\f$ \infty \f$</sub>-Norm, to calculate the maximum or minimum of vector. Sometimes one wants to integrate
a DOFVector along a curve. There are some classical reduction algorithms implemented and several methods to integrate a
DOFVector or an AbstractFunction over a mesh (e.g. \f$ \Omega \f$) or a subdomain (e.g. \f$ \Gamma_i \f$), for example:

\code
u.Int(); // \int_\Omega u(x) dx
u.IntOnBoundary(i); // \int_{\Gamma_i} u(x) dx
u.L1Norm(); // \int |u(x)| dx
u.L2Norm(); // sqrt( \int u(x)^2 dx )
u.H1Norm(); // sqrt( \int u(x)^2 + |grad(u(x))|^2 dx )
u.min(); // min_i(u[i])
u.max(); // max_i(u[i])
u.sum(); // sum_i u[i]
u.absMax(); // max_i(|u[i]|)
u.average(); // 1/size(u) sum_i u[i]
\endcode

More advanced integrate methods concern more than one DOFVector, e.g. integrate over the product of two DOFVectors, or 
over a function applied to the values of a DOFVector. As an example we want to calculate the integral of a DOFVector 
over the interface of a circle (or a ball in 3d), by integration over an approximation of a surface delta function. 
This approximation is given by the double-well defined in the last section. At first we want to integrate the 
function \f$ f(x)=1 \f$, i.e. we want to calculate the area and the interface length, then the surface integral of u:

\code
DOFVector< double > p(feSpace, "p");
DOFVector< double > B(feSpace, "B(p)");

double area = integrate_Coords(feSpace, new PhaseField);

p.interpol(new PhaseField);
double perimeter = integrate_Vec(p, new DoubleWell);

transformDOF(&p, &B, new DoubleWell);
double intOnCircle = integrate_Vec2(p, B, new AMDiS::Mult< double >);
\endcode

A list of integration methods can be found in DOFVector.h:
- integrate_Coords(feSpace, Functor) = \f$ \int f(x)\,dx \f$
- integrate_Vec(DOFVector, (unary) Functor) = \f$ \int f(v)\,dx \f$
- integrate_Vec2(DOFVector1, DOFVector2, (binary) Functor)  = \f$ \int f(v_1, v_2)\,dx \f$
- integrate_VecTimesCoords(DOFVector, Functor) = \f$ \int v\cdot f(x)\,dx \f$
- integrate_VecAndCoords(DOFVector, (binary) Functor) = \f$ \int f(v, x)\,dx \f$
- integrateGeneral(...) = \f$ \int f(\{v_i\}_i)\,dx \f$
- integrateGeneralGradient(...) = \f$ \int f(\{v_i\}_i, \{grad(v_i)\}_i)\,dx \f$

\subsection integrate_on_subdomains Integrate on subdomains

Sometims you want to integrate over parts of the geometry, e.g. over (parts of) the boundary of the domain or a real subdomain. This 
can be done using integrate methods defined in MeshIndicator.h. The idea is to add an additional argument to the integrate functions 
that indicates where to integrate. This argument is a MEshIndicator object:

\code
struct MeshIndicator
{
  MeshIndicator(WorldVector<double> dimension);
  MeshIndicator(Mesh* mesh);

  bool operator()(const WorldVector<double>& x) const;
  bool operator()(const ElInfo* elInfo) const;
  ...
}
\endcode

Two types of indicators can be implemented, a coords based indicator and an element based indicator. If just the coords based indicator
is given, in <c>operator()(const ElInfo* elInfo)</c> it is tested whether all nodes of the element give true in the 
<c>operator()(const WorldVector<double>& x)</c>.

In the example, how to use a MeshIndicator, we want to integrate over a disk:

\code
struct DiskIndicator : MeshIndicator
{
  DiskIndicator(Mesh* mesh_, double radius_) : MeshIndicator(mesh_), radius(radius_) {};

  bool operator()(const WorldVector<double>& x) const
  {
    return norm(x) <= radius;
  }
  
private:
  double radius;
}
...
double disk_area = tools::integrate(u, new DiskIndicator(mesh, 0.3);
\endcode

You have to be carefull, using the MeshIndicator, since it just filters the elements that are indicated as false and integrates over all other
elements, so if the mesh is too coarse the integration over curved areas gives only a coarse approximation. A more accurate integration is
implemented in the toolbox compositeFEM. In the file CFE_Integration.h the static methods <c>integrate_onNegLs</c> and <c>integrate_onZeroLs</c> 
are appropriate. The first one calculates an integral of a function f on a domain where a level set function is negative and the second one 
calculates the surface integral of a function f on the zero level set.

\code
CFE_Integration::integrate_onNegLs(ElementFunction<double> *f, ElementLevelSet *elLS, ...)
CFE_Integration::integrate_onZeroLs(ElementFunction<double> *f, ElementLevelSet *elLS, ...)
\endcode

We want to repeat the calculation of a disk area and perimeter, using the compositeFEM functions:

\code
ElementFunctionAnalytic<double> oneFct(new AMDiS::Const<double, WorldVector<double> >(1.0));
ElementFunctionAnalytic<double> elLevelFct(new Circle(radius, center));
ElementLevelSet elLevelSet("circle", &elLevelFct, mesh);

double disk_area = CFE_Integration::integrate_onNegLs(&oneFct, &elLevelSet);
double disk_perimeter = CFE_Integration::integrate_onZeroLs(&oneFct, &elLevelSet);
\endcode

\section gradient_of_dofs Gradients of DOFVectors

The task to calculate derivates of DOFVectors occures very often, for example in the calculation of a stress tensor or if the normal vector 
of an implicit interface is calculated via the gradient of the implicit function. There are tree ways to calculate gradients at the moment:
- DOFVector\f$\rightarrow\f$getGradient(...)
- DOFVector\f$\rightarrow\f$getRecoveryGradient(...)
- weak gradients using a differential equation

All three methods are introduced here. The first one is a fast, but coarse approximation of the gradient. It traverses over the mesh and
assigns to each degree of freedom the gradient value of one adjacent element. In an element the gradient can be calculated directly from the 
base functions. The second methods is more accurate and adopts the ideas of gradient recovery techniques. In some points of the elements
the approximation of the gradients are of higher order. This points are called super convergent point, for example the centroids of the 
elements. The recovery gradient calculates for all degrees of freedom the gradients in the centroids of adjacent elements and averages over
this values weighted by the area of the elements.

\code
DOFVector< WorldVector<double> > grad0(feSpace, "simple_gradient");
DOFVector< WorldVector<double> > grad1(feSpace, "recovery_gradient");
u.getGradient(&grad0);
u.getRecoveryGradient(&grad1);
\endcode

To calculate the gradient in weak form a very simple system of equation must be set up: \f$ v_i = \partial_i u \f$

\code
ProblemStat prob("gradient");
prob.initialize(INIT_ALL);

WorldVector< DOFVector<double>* > grad_weak;

for (size_t i = 0; i < Global::getGeo(WORLD); i++) {
  grad_weak[i] = new DOFVector<double>(prob.getFeSpace(i), "grad_"+boost::lexical_cast<std::string>(i));
  
  Operator* vi = new Operator(prob.getFeSpace(i), prob.getFeSpace(i));
  vi->addTerm(new Simple_ZOT);
  prob.addMatrixOperator(*vi, i, i);
  
  Operator* grad_ui = new Operator(prob.getFeSpace(i));
  grad_ui->addTerm(new PartialDerivative_ZOT(&u, i));
  prob.addVectorOperator(*grad_ui, i);
}

AdaptInfo adaptInfo("adapt", prob.getNumComponents());
AdaptStationary adapt("adapt", prob, adaptInfo);
adapt.adapt(); // start the calculation
\endcode

The operator <c>PartialDerivative_ZOT</c> is defined in POperators_ZOT.h.

\subsection sym_gradients Symmetric gradients

To readuce the complexity of simulations sometimes it is possible to take advantage of symmetric properties of the problem, e.g.
in elasticity of dendrite growth. IF you have to calculate gradients near the symmetry axis you get problems, since the gradient 
approximation are very bad, especially the homogeneous neumann condition, i.e. \f$ \partial_n u = 0 \f$, can often not be recovered.
The method <c>getRecoveryGradientSym(...)</c> overcomes this problem, by mirroring the gradient at the neumann boundary. This
method is implemented in the file GradientCalculations.h:

\code
DOFVector< WorldVector<double> > grad2(feSpace, "symmetric_gradient");
getRecoveryGradientSym(&u, &grad2, true);
\endcode

\section eval_dof_vectors Evaluation of DOFVectors at coordinates

As in the introduction mentioned the evaluation of a DOFVector at coordinates is an expensive task. But AMDiS has a hierarchical mesh
structure that can improve the evaluation. For a standard DOFVector u one can call the method

\code
DOFVector< T > u(feSpace, "u");
// coordinate where to evaluate the DOVector
WorldVector<double> p; // e.g. p[0] = 0.2; p[1] = 0.3;
T value = u.evalAtPoint(p);
\endcode

For a special container, called <c>DOFView</c>, the evaluation can also be simplified to

\code
DOFView u2(u);
T value = u2[p];
\endcode

Or for several Container types, just call

\code
T value1 = evalAtPoint(u, p);
T value2 = evalAtPoint(u2, p);
\endcode

A standard task is postprocessing is the evaluation of a DOFVector over a line (like in ParaView plotOverLine). In the Helpers.h this is
implemented for Container-types, like DOFVector, DOFView, AbstractFunction, and can be called with the method

\code
template<typename Container>
void interpolOverLine(const Container &vec, 
		      WorldVector<double> xMin, WorldVector<double> xMax, 
		      int nPoints,
		      std::vector< WorldVector<double> > &x, std::vector< double > &y);
...
// define coordinates: [0,0]->[1,1]
WorldVector<double> xMin; xMin.set(0.0);
WorldVector<double> xMax; xMax.set(1.0);
// vectors for the result
std::vector< WorldVector<double> > x;
std::vector< double > y;
// start the interpolation
interpolOverLine(u, xMin, xMax, 1000, x, y);
\endcode


*/

//-----------------------------------------------------------


/*! \page mesh_refinement Mesh refinement tools

The AMDiS-Extension provide a powerful tool to local mesh refinement. You just have
to give the information where to refine the mesh and how fine the mesh should be refined. 
This can be done in several way and on the base of different values, e.g.

- MeshRefinementFunctions
- AbstractFunctions
- DOFVector

that can be processed to calculate the local refinement level.

\section mesh_refinement_function MeshRefinementFunctions

The \a MeshRefinementFunction is defined in the file Refinement.h. The most imported methods are shown here

\code
template<typename Tin, typename Tout>
struct MeshRefinementFunction : AbstractFunction<Tout, Tin>
{
  MeshRefinementFunction(Mesh* mesh_);
  virtual Tout operator()(const Tin &value) const;
  ...
}
\endcode

The template arguments have the following meaning: 

- <b>Tout</b>: <c>double</c> for mesh size, <c>int</c> for refinement level
- <b>Tin</b>: <c>WorldVector&lt;double&gt;</c> for centroid  coordinates of mesh elements, or <c>double</c>
	      for the values of an input DOFVector at the centroid coordinates of an element.

The contructor gets a pointer to the mesh, because it calculates the size of the macro-elements, so that the 
class can provide methods to covert between refinement level and element size. The operator <c>operator()</c> 
is called in mesh traversal for element marking for refinement.

The first example using just this MeshRefinementFunction implements local refinement around given points with
a given ball radius.

\code
struct CoordsRefinement : MeshRefinementFunction< WorldVector<double>, int >
{
  CoordsRefinement(std::string name, Mesh* mesh, std::vector< WorldVector<double> > points, double radius) 
  ...
}
\endcode

The class gets a name (for parameter specification in the init-file), the Mesh pointer, a vector of point coordinates
and the refinement radius. You can define two parameters in the init-file:

<c>
[name]->level on points [=20]<br />
[name]->level in inner domain [=15]
</c>

To perform the refinement you have to put this refinement function into the class \a RefinementLevelCoords, defined in Refinement_Level.h.
This class gets a finite element space and the refinement function in the constructor:

\code
struct RefinementLevelCoords : RefinementLevel< WorldVector<double>, int >
{
  RefinementLevelCoords(const FiniteElemSpace* feSpace, MeshRefinementFunction< WorldVector<double>, int >* refinement_function)
  ...
}
\endcode

The following code snippet reads a list of points from the init-file and refines the mesh around these points with the help of the mesh 
refinement functions introduced above:

\code
int main(int argc, char** argv)
{
  AMDiS::init(argc, argv);
  
  // initialize the problem
  ProblemStat prob("prob");
  prob.initialize(INIT_ALL);
  
  // refine the mesh
  int nPoints = 1;
  Parameters::get("refinement->num points", nPoints);
  std::vector<WorldVector<double> > points(nPoints);
  for (size_t i = 0; i < nPoints; i++)
    Parameters::get("refinement->point["+boost::lexical_cast<std::string>(i)+"]", points[i]);
  double radius = 0.1;
  Parameters::get("refinement->radius", radius);
  
  CoordsRefinement refFunction("refinement", prob.getMesh(), points, radius);
  RefinementLevelCoords refinement(prob.getFeSpace(), &refFunction);

  refinement.refine(10);
  
  // Adapt-Infos
  AdaptInfo adaptInfo("adapt", prob.getNumComponents());  
  prob.writeFiles(adaptInfo, false);
  
  AMDiS::finalize();
  return 1;
};
\endcode

The result can be seen at the following screenshot:
\image html mesh_refinement2.png "Mesh structure with refined points in the lower left and upper right corner" width=300px

\subsection abstract_function AbstractFunctions

The second example implements a refinement at the zero level set \f$ \Gamma \f$ of a signed distance function \f$ \phi \f$:
\f[\Gamma:=\{x\in\mathbf{R}^{n}:\;\phi(x)=0\}\f]
The function \f$ \phi(x) \f$ is defined as <c>AbstractFunction&lt; double, WorldVector&lt;double&gt; &gt;</c>. There are
a lot of examples of signed distance functions defined in the file SignedDistFunctors.h, e.g. \a Circle, \a Ellipse, \a Polygon.

To define where to refine and up to wich level an other MeshRefinementFunction must be implemented, e.g.

\code
struct SignedDistRefinement : MeshRefinementFunction< double, int >
{
    SignedDistRefinement(std::string name_, Mesh* mesh_);
    int operator()(const double& dist) const;
    ...
}
\endcode

The class gets a name (for parameter specification in the init-file) and the Mesh pointer. You can define several parameters in the init-file:

<c>
[name]->level in inner domain [=10]<br />
[name]->level in outer domain [=10]<br />
[name]->level on interface [=14]<br />
[name]->initial level [=10]<br />
[name]->interface width [=0.2]<br />
[name]->fade out width [=0.0]<br />
[name]->sign in inner domain [=-1]
</c>

To perform the refinement you have to put this refinement function into the class \a RefinementLevelCoords2, defined in Refinement_Level.h.
This class gets a finite element space and the refinement function and a pointer to an AbstractFunction for the signed distance function
in the constructor:

\code
struct RefinementLevelCoords2 : RefinementLevel< double, int >
{
  RefinementLevelCoords2(const FiniteElemSpace* feSpace, 
			 MeshRefinementFunction< double, int >* refinement_function, 
			 AbstractFunction< double, WorldVector<double> >* signed_distance_function);
  ...
}
\endcode

The following code snippet reads parameters for a signed distance function, defining a circle, from the init-file and refines the mesh 
around the interface level with the help of the mesh refinement functions introduced above:

\code
int main(int argc, char** argv)
{
  AMDiS::init(argc, argv);

  // initialize the problem
  ProblemStat prob("prob");
  prob.initialize(INIT_ALL);
  
  // refine the mesh
  double radius = 0.3;
  WorldVector<double> center; center.set(0.5);
  Parameters::get("refinement->radius", radius);
  Parameters::get("refinement->center", center);
  
  SignedDistRefinement refFunction("refinement", prob.getMesh());
  RefinementLevelCoords2 refinement(prob.getFeSpace(), &refFunction,
      new Circle(radius, center));

  refinement.refine(10);

  // Adapt-Infos
  AdaptInfo adaptInfo("adapt", prob.getNumComponents());  
  prob.writeFiles(adaptInfo, false);
  
  AMDiS::finalize();
  return 1;
};
\endcode

The result can be seen at the following screenshot:
\image html mesh_refinement1.png "Mesh structure with refined circle"

\subsection dof_vectors DOFVectors

The third example implements a refinement around the interface of a phasefield function, given by a DOFVector. This phasefield function
occures as solution of an Allan-Cahn, or Cahn-Hilliard equation, for example. A definition of a phasefield function can be found in
the page about \subpage data_on_unstructured_grid data on unstructured grids.

To define where to refine and up to wich level a MeshRefinementFunction must be implemented, e.g.

\code
struct PhaseFieldRefinement : MeshRefinementFunction< double, int >
{
    PhaseFieldRefinement(std::string name_, Mesh* mesh_);
    int operator()(const double& phase) const;
    ...
}
\endcode

The class gets a name (for parameter specification in the init-file) and the Mesh pointer. You can define several parameters in the init-file:

<c>
[name]->level in inner domain [=10]<br />
[name]->level in outer domain [=10]<br />
[name]->level on interface [=14]<br />
[name]->initial level [=10]<br />
[name]->min interface value [=0.05]<br />
[name]->max interface value [=0.95] <br />
[name]->min outer interface value [=0.001]<br />
[name]->max outer interface value [=0.999]<br />
</c>

At first a global refinement up to the <c>initial level</c> is performed, then the local refinement. If you specify values 
<c>min outer interface value != min interface value</c>, or <c>max outer interface value != max interface value</c>, you can define a fade out
region. Otherwise the interface is refined in the phasefield value interval <c>[min interface value, max interface value]</c>, by default 
in [0.05, 0.95].

To perform the refinement you have to put this refinement function into the class \a RefinementLevelDOF, defined in Refinement_Level.h.
This constructor gets a finite element space and the refinement function and a pointer to a DOFVector that holds the values, that are passed
to the MeshRefinementFunction:

\code
struct RefinementLevelDOF : RefinementLevel< double, int >
{
  RefinementLevelDOF(const FiniteElemSpace* feSpace_, 
		     MeshRefinementFunction< double, int >* refinement_function_, 
		     DOFVector<double>* dof_vector);
  ...
}
\endcode

The following code snippet reads parameters for a signed distance function, defining a circle, from the init-file, creates a phasefield
DOFVector and refines the mesh around the interface with the help of the mesh refinement functions introduced above:

\code
int main(int argc, char** argv)
{
  AMDiS::init(argc, argv);

  // initialize the problem
  ProblemStat prob("prob");
  prob.initialize(INIT_ALL);
  
  // refine the mesh
  double radius = 0.3;
  WorldVector<double> center; center.set(0.5);
  Parameters::get("refinement->radius", radius);
  Parameters::get("refinement->center", center);
  
  DOFVector< double > phasefield(prob.getFeSpace(), "phasefield");
  phasefield.set(1.0);
  
  PhaseFieldRefinement refFunction("refinement", prob.getMesh());
  RefinementLevelDOF refinement(prob.getFeSpace(), &refFunction, &phasefield);

  for (int i = 0; i < 5; i++) {
    refinement.refine(4);
    phasefield.interpol(new SignedDistFctToPhaseField(epsilon, new Circle(radius, center), 3.0);
  }

  // Adapt-Infos
  VtkWriter::writeFile(phasefield, "phasefield.vtu");
  
  AMDiS::finalize();
  return 1;
};
\endcode

The class <c>SignedDistFctToPhaseField</c> is defined in the file PhaseFieldConvert.h and converts (as the name already indicates)
a signed distance function into a phasefield function. In this example you have to loop over refinement and interpolation, since
the interpolated data (that indicates where to refine the mesh) lives on the actual mesh, that is coarse at the beginning. In the first
refinement.refine(...) call the mesh is globally refined up to the <c>initial level</c> specified in the init-file, so that the phasefield
function and especially the interface can be represented by the mesh.

*/

//-----------------------------------------------------------


/*! \page boundary_conditions Boundary conditions

For a second order elliptic partial differential equation one has to describe the geometry and prescribe boundary conditions to close
the system. Three types of boundary conditions are very typical: Dirichlet-type, Neumann-type or the Robin-type boundary condition and 
the periodic boundary conditions. AMDiS provides interfaces for all these types. In the macro-mesh one has to mark several edges as 
boundary edges. During refinement the mark is handed down to the leaf elements. That's the crux of the matter. Boundary conditions are
defined edge-wise, but what to do if two edges with different boundary conditions collide? In the macro-mesh one has to set for each
separate boundary part a boundary number. Depending on this number the one or other condition is preferred for the colliding vertex.
For problems with more than one equation on the same macro-mesh one can construct examples that are solvable with this strategy. Also
there are problem with the periodic boundary condition and mixed finite elements. Neumann boundary conditions in complicated parts of
a boundary need also special consideration. Here some new techniques are introduced to overcome some of the problems.
- Dirichlet boundary conditions
  - singular points
  - implicit areas
- Periodic boundary conditions
- Implicit Neumann condition

\section dirichlet_bc Dirichlet boundary conditions
There are three methods to enforce Dirichlet boundary condition, i.e. \f$ u = g\; on\; \Gamma \f$. To use these you have to replace the 
standard ProblemStat by ExtendedProblemStat. The methods <c>addSingularDirichletBC(...)</c> add a condition at only one single degree
of freedom. For the second method <c>addImplicitDirichletBC(...)</c> you have to pass a function, that has negative values in the 
vertices where to enforce the boundary condition, e.g. a signed distance function. The last method <c>addManualDirichletBC(...)</c>
provides an interface to describe parts of a boundary easily.

\subsection singular_dirichlet_bc Singular Dirichlet boundary condition

For example we want to solve the Neumann problem, we have to define an additional condition, since \f$ u \f$ is only defined up to a constant.

\f[ \Delta u = f\quad in\; \Omega,\quad\partial_n u = 0\quad on\; \partial\Omega \f]

One way out of this dilemma is to set a Dirichlet boundary condition on just one point. There are two ways to do this: first you can give a
dofindex for which to set the value, second you can specify a coordinate and the algorithm looks for the nearest degree of freedom to set
the boundary condition there.

\code
template<typename ValueContainer>
  void addSingularDirichletBC(DegreeOfFreedom idx, int row, int col,
			      ValueContainer &values);
template<typename ValueContainer>
  void addSingularDirichletBC(WorldVector<double> &pos, int row, int col,
			      ValueContainer &values);
\endcode

You have to the row and column where to set the boundary condition in the system of differential equations and as last argument you have to give
the value that should be enforced. The <c>ValueContainer</c> can ether be scalar reference, i.e. <c>double&amp;</c>, an 
<c>AbstractFunction&lt; double, WorldVector&lt;double&gt; &gt;</c>, or a <c>DOFVector&lt; double &gt;</c>.

The next example shows how to use this methods:

\code
int main(int argc, char* argv[])
{
  AMDiS::init(argc, argv);

  // ===== create and init the scalar problem ===== 
  ExtendedProblemStat ellipt("ellipt");
  ellipt.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo adaptInfo("ellipt->adapt", ellipt.getNumComponents());
  AdaptStationary adapt("ellipt->adapt", ellipt, adaptInfo);
  
  // ===== create matrix operator =====
  Operator matrixOperator(ellipt.getFeSpace());
  matrixOperator.addTerm(new Simple_SOT);
  ellipt.addMatrixOperator(matrixOperator, 0, 0);

  // ===== create rhs operator =====
  Operator rhsOperator(ellipt.getFeSpace());
  rhsOperator.addTerm(new Simple_ZOT(1.0));
  ellipt.addVectorOperator(rhsOperator, 0);

  // ===== add singular dirichlet boundaty condition =====
  WorldVector<double> p; p.set(0.0);
  double zero = 0.0;
  ellipt.addSingularDirichletBC(p, 0, 0, zero);

  // ===== start simulation =====
  adapt.adapt();
  ellipt.writeFiles(adaptInfo, true);

  AMDiS::finalize();
}
\endcode

The solution will look like
\image html elliptSingular.png "Singular Dirichlet condition in the bottom left corner" width=300px

\subsection implicit_dirichlet_bc Implicit Dirichlet boundary condition

Assume you want to solve an equation in a "complicated" domain, that can be described by a signed distance function. Then
you can use a Diffuse-Domain approach or the implicit dirichlet boundary conditions defined in ExtendedProblemStat.h. Two
interfaces are available. You can set a DOFVector that holds the values of the signed distance function, or you can specify
an AbstractFunction:

\code
template<typename ValueContainer>
  void addImplicitDirichletBC(DOFVector<double> &signedDist,
			      int row, int col,
			      ValueContainer &values);

template<typename ValueContainer>
  void addImplicitDirichletBC(AbstractFunction<double, WorldVector<double> > &signedDist,
			      int row, int col,
			      ValueContainer &values);
\endcode

Similar to the singular Dirichlet conditions the value container can be a scalar, an AbstractFunction, or a DOFVector. In the next Example
the signed distance functions describes a disk area. At the boundary the Dirichlet condition should be enforced. The algorithm now not only
set the Dirichlet values at the boundary, but in the whole area with distance values less than zero. So a simple algebraic equation is solved
in the outer part of the complicated domain. Near the boundary the mesh should be refined, since one wants to resolve the boundary vertices 
very near to the implicit interface.

\code
int main(int argc, char* argv[])
{
  AMDiS::init(argc, argv);

  // ===== create and init the scalar problem ===== 
  ExtendedProblemStat ellipt("ellipt");
  ellipt.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo adaptInfo("ellipt->adapt", ellipt.getNumComponents());
  AdaptStationary adapt("ellipt->adapt", ellipt, adaptInfo);

  // ===== signedDist function that describes the geometry =====
  AbstractFunction<double, WorldVector<double> > *signedDistFct = new InverseCircle(1.0);

  // ===== refine mesh at interface =====
  SignedDistRefinement refFunction(ellipt.getMesh());
  RefinementLevelCoords2 refinement(
    ellipt.getFeSpace(),
    &refFunction,
    signedDistFct);
  refinement.refine(9);
  
  // ===== create matrix operator =====
  Operator matrixOperator(ellipt.getFeSpace());
  matrixOperator.addTerm(new Simple_SOT);
  ellipt.addMatrixOperator(matrixOperator, 0, 0);

  // ===== create rhs operator =====
  Operator rhsOperator(ellipt.getFeSpace());
  rhsOperator.addTerm(new Simple_ZOT(1.0));
  ellipt.addVectorOperator(rhsOperator, 0);

  // ===== add boundary conditions =====  
  double zero = 0.0;
  ellipt.addImplicitDirichletBC(*signedDistFct, 0, 0, zero);

  // ===== start simulation =====
  adapt.adapt();
  ellipt.writeFiles(adaptInfo, true);

  AMDiS::finalize();
}
\endcode

The solution is shown in the next pictures. On the left one can see the mesh locally refined on the boundary of a circle:
\image html elliptImplicit.png "Implicit Dirichlet condition at the boundary of a circle" width=300px

\subsection manual_dirichlet_bc Dirichlet boundary condition with mesh indicators

How to solve the problems of AMDiS with Dirichlet condition, with the priority of one condition over the other? How to enforce the essential
boundary conditions independent of the boundary number set in the macro-mesh? The answer is to use manual Dirichlet boundary conditions, also
defined in ExtendedProblemStat.h. There you do not (necessarily) describe the boundary by a boundary number, but by a boundary indicator.

A mesh- or boundary indicator is an AbstractFunction, that return true if the coordinate, where it is evaluated, lies on the desired boundary,
and false otherwise. (Probably it would be better here to use the struct MeshIndictor that has exactly the desired structure). Optionally you can give
also a boundary number so that the combination of both describes the boundary part for the Dirichlet condition:

\code
template<typename ValueContainer>
  void addManualDirichletBC(AbstractFunction<bool, WorldVector<double> >* meshIndicator,
			    int row, int col,
			    ValueContainer &values);
template<typename ValueContainer>
  void addManualDirichletBC(BoundaryType nr, AbstractFunction<bool, WorldVector<double> >* meshIndicator,
			    int row, int col,
			    ValueContainer &values);		    
\endcode
			    
In the next example we want to enforce a Dirichlet boundary condition on the left boundary, that has the x-coordinate 0.0:

\code
struct LeftBoundary : AbstractFunction< bool, WorldVector<double> >
{
  bool operator()(const WorldVector<double>& x) const
  {
    return x[0] < DBL_TOL;
  }
};

int main(int argc, char* argv[])
{
  AMDiS::init(argc, argv);

  // ===== create and init the scalar problem ===== 
  ExtendedProblemStat ellipt("ellipt");
  ellipt.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo adaptInfo("ellipt->adapt", ellipt.getNumComponents());
  AdaptStationary adapt("ellipt->adapt", ellipt, adaptInfo);
  
  // ===== create matrix operator =====
  Operator matrixOperator(ellipt.getFeSpace());
  matrixOperator.addTerm(new Simple_SOT);
  ellipt.addMatrixOperator(matrixOperator, 0, 0);

  // ===== create rhs operator =====
  Operator rhsOperator(ellipt.getFeSpace());
  rhsOperator.addTerm(new Simple_ZOT(1.0));
  ellipt.addVectorOperator(rhsOperator, 0);

  // ===== add boundary conditions ===== 
  double one = 1.0;
  ellipt.addManualDirichletBC(new LeftBoundary, 0, 0, one);

  // ===== start simulation =====
  adapt.adapt();
  ellipt.writeFiles(adaptInfo, true);

  AMDiS::finalize();
}
\endcode

\image html elliptManual.png "Manual Dirichlet condition at the left boundary of the domain" width=300px

*/

//-----------------------------------------------------------


/*! \page base_problems Base Problems

Some common problems are implemented in the directory base_problems:
- \subpage cahn_hilliard
- \subpage phase_field_crystal
- \subpage navier_stokes
- \subpage chns
- \subpage chns2
- \subpage linear_elasticity
- \subpage fsi

*/

//-----------------------------------------------------------


/*! \page fsi Fully coupled fluid structure interaction using diffuse domain approach

The underliing equations are

\f{eqnarray*}{
  \partial_t u_j + \mathbf{u}\cdot\nabla u_j - \nabla\cdot(\phi\cdot\sigma_f) - \nabla\cdot((1-\phi)\cdot\sigma_s) &=& 0,\quad j=0\ldots d-1 \\
  \nabla\cdot(\phi\cdot\mathbf{u}) - \nabla\phi \cdot \mathbf{u} + (1-\phi)(p - \alpha\nabla\cdot\eta) &=& 0 \\
  (1-\phi)\cdot(\partial_t\eta_j + \mathbf{u}\cdot\nabla\eta_j) &=& (1-\phi)\cdot u_j,\quad j=0\ldots d-1
\f}

with \f$ \alpha = \lambda + \frac{2}{3}\mu,\; \sigma_s = \mu(\nabla\eta+\nabla\eta^\top)+\lambda I\nabla\cdot\eta 
\f$ and \f$ \sigma_f = \nu(\nabla \mathbf{u} + \nabla\mathbf{u}^\top)-Ip \f$.

*/

//-----------------------------------------------------------


/*! \page chns Cahn-Hilliard-Navier-Stokes equation

Two phase flow simulations are often implemented using by coupling the Cahn-Hilliard equations and the 
Navier-Stokes equations. This coupling can be done explicitely, i.e. solve the equations one after the other,
or implicitely as one system of equations. This is implemented in the CahnHillirdNavierStokes base problem:

\f{eqnarray*}{
  \partial_t u_j + \mathbf{u}\cdot\nabla u_j - \nabla\cdot\sigma &=& \mu\partial_j c,\quad j=0\ldots d-1 \\
  \nabla\cdot \mathbf{u} &=& 0 \\
  \partial_t c - \Delta\mu + \mathbf{u}\cdot\nabla c &=& 0 \\
  -\epsilon^2\Delta c + W'(c) &=& \mu
\f}

with \f$ \sigma = Ip - \nu\nabla u \f$ and \f$ W(c) = \frac{1}{4}c^2(1-c)^2 \f$.

For the time-discretization the nonlinear terms are linearized around the old timestep:

\f{eqnarray*}{
  \frac{1}{\tau} (u^{k+1}_j - u^{k}_j) + \mathbf{u}^{k+1}\cdot\nabla u^k_j + \mathbf{u}^{k}\cdot\nabla u^{k+1}_j - \mathbf{u}^{k}\cdot\nabla u^{k}_j - \nabla\cdot(\sigma^{k+1}) &=& \mu^{k+1}\partial_j c^k + \mu^{k}\partial_j c^{k+1} - \mu^{k}\partial_j c^k,\quad j=0\ldots d-1 \\
  \nabla\cdot \mathbf{u}^{k+1} &=& 0 \\
  \frac{1}{\tau} (c^{k+1} - c^k) - \Delta\mu^{k+1} &=& - \mathbf{u}^{k+1}\cdot\nabla c^k - \mathbf{u}^{k}\cdot\nabla c^{k+1} + \mathbf{u}^{k}\cdot\nabla c^k \\
  \mu^{k+1} &=& -\epsilon^2\Delta c^{k+1} + W'(c^k) + W''(c^k)(c^{k+1}-c^k)
\f}

*/

//-----------------------------------------------------------


/*! \page chns2 Cahn-Hilliard-Navier-Stokes equation (variable density/viscosity)

Two phase flow simulations are often implemented using by coupling the Cahn-Hilliard equations and the 
Navier-Stokes equations. This coupling can be done explicitely, i.e. solve the equations one after the other,
or implicitely as one system of equations. This is implemented in the CahnHillirdNavierStokes base problem:

\f{eqnarray*}{
  \partial_t c + \mathbf{u}\cdot\nabla c &=& \nabla\cdot \gamma W(c) \nabla \mu \\
  0 &=& \mu + \epsilon^2\Delta c - W'(c) \\
  \rho(c)\partial_t u_j + \rho(c)\mathbf{u}\cdot\nabla u_j &=& \nabla\cdot\nu(c)\nabla u_j - \nabla p + \alpha\mu\partial_j c + F_j,\quad j=0\ldots d-1 \\
  0 &=& \nabla\cdot \mathbf{u}
\f}

with \f$ W(c) = \frac{1}{4}c^2(1-c)^2 \f$ and \f$ \alpha \f$ the surface tension.

\section chns2_euler Semi-implicit Euler discretization

For the time-discretization a semi-implicit euler discretization is chosen and the nonlinear terms are linearized around the old timestep:

\f{eqnarray*}{
  \frac{1}{\tau} (c^{k+1} - c^k) - \Delta\mu^{k+1} &=& - \mathbf{u}^{k+1}\cdot\nabla c^k - \mathbf{u}^{k}\cdot\nabla c^{k+1} + \mathbf{u}^{k}\cdot\nabla c^k \\
  \mu^{k+1} &=& -\epsilon^2\Delta c^{k+1} + W'(c^k) + W''(c^k)(c^{k+1}-c^k) \\
  \rho(c^k)\frac{1}{\tau} (u^{k+1}_j - u^{k}_j) + [\rho(c)\mathbf{u}\cdot\nabla u_j]  &=& \nabla\cdot(\nu(c^k)\nabla u_i^{k+1}) - \nabla p^{k+1} + \alpha(\mu^{k+1}\partial_j c^k + \mu^{k}\partial_j c^{k+1} - \mu^{k}\partial_j c^k) + F_i,\quad j=0\ldots d-1 \\
  \nabla\cdot \mathbf{u}^{k+1} &=& 0
\f}

with \f[ [\rho(c)\mathbf{u}\cdot\nabla u_j] := \rho(c^k)\cdot\mathbf{u}^{k+1}\cdot\nabla u^k_j + \rho(c^k)\cdot\mathbf{u}^{k}\cdot\nabla u^{k+1}_j + c^{k+1}\cdot\rho'(c^k)\cdot\mathbf{u}^{k}\cdot\nabla u^{k}_j - \big(\rho(c^k) + c^k\cdot\rho'(c^k)\big)\cdot\mathbf{u}^{k}\cdot\nabla u^{k}_j \f].

\section chns2_rb Rosenbrock Discretization

For the time-discretization a Rosenbrock scheme is chosen. 
Therefor we have to introduce a new variable \f$ z_j = \partial_t u_j + \mathbf{u}\cdot\nabla u_j \f$.
Stage solutions are depicted with supscripts \f$ \cdot^s \f$ and old solution with \f$ \cdot^{old} \f$:

\f{eqnarray*}{
  \partial^s_t c - \Delta\mu + \mathbf{u}\cdot\nabla c^{old} + \mathbf{u}^{old}\cdot\nabla c &=& \Delta\mu^s - \mathbf{u}^{s}\cdot\nabla c^s \\
  -\mu - \epsilon^2\Delta c + W''(c^{old})c&=& \mu^s + \epsilon^2\Delta c^{s} - W'(c^s) \\
  \partial^s_t u_j - z_j + u^{old}\cdot\nabla u_j + u\cdot\nabla u^{old}_j &=& z^s - u^s\cdot\nabla u^s_j,\quad j=0\ldots d-1 \\
  \rho(c^{old})z + \rho'(c^{old})z^{old}\cdot c - \nabla\cdot\nu(c^{old})\nabla u_j - \nabla\cdot[c\nu'(c^{old})\nabla u^{old}_j] \\
  + \nabla p - \alpha\mu\partial_j c^{old} - \alpha\mu^{old}\partial_j c &=&  \nabla\cdot\nu(c^s)\nabla u^s_j - \nabla p^s + \alpha\mu^{s}\partial_j c^s + F_j,\quad j=0\ldots d-1 \\
  -\nabla\cdot \mathbf{u} &=& \nabla\cdot \mathbf{u}^s
\f}

*/

#endif // AMDIS_EXTENSIONS_TUTORIAL_INCLUDE
